//=============================================================================
// "Arduino" example program for Crystalfontz ePaper. 
//
// This project is for the CFAP122250A0-0213 :
//
//   https://www.crystalfontz.com/product/cfap122250a00213
//
// It was written against a Seeduino v4.2 @3.3v. An Arduino UNO modified to
// operate at 3.3v should also work.
//-----------------------------------------------------------------------------
// This is free and unencumbered software released into the public domain.
// 
// Anyone is free to copy, modify, publish, use, compile, sell, or
// distribute this software, either in source code form or as a compiled
// binary, for any purpose, commercial or non-commercial, and by any
// means.
// 
// In jurisdictions that recognize copyright laws, the author or authors
// of this software dedicate any and all copyright interest in the
// software to the public domain. We make this dedication for the benefit
// of the public at large and to the detriment of our heirs and
// successors. We intend this dedication to be an overt act of
// relinquishment in perpetuity of all present and future rights to this
// software under copyright law.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// 
// For more information, please refer to <http://unlicense.org/>
//=============================================================================
// The display is SPI, include the library header.
#include <SPI.h>

// Include the ePaper module header.
#include "ePaper_CFAP122250A00213.h"

// Include the images. These images were prepared with "bmp_to_epaper" which
// is available on the Crystalfontz site.
#include "Images_for_CFAP122250A00213.h"
// This module is 122x250 pixels. Make sure that the image files make sense.
#if ((WIDTH_PIXELS != (122)) || (HEIGHT_PIXELS  != (250)))
#error "IMAGE INCLUDE FILE IS WRONG SIZE"
#endif


//=============================================================================
void Partial_Update_Letters_Demo(void)
  {
  uint8_t
    i;

  //Load the background inage into both layers of the display
  //controller's RAM
  Partial_Update_Init_Background(WIDTH_PIXELS,  //X size
                                 HEIGHT_PIXELS, //Y size
                                 Image_0);      //image data

  // Now we can animate some smaller bitmaps to show
  // the four letters 'C' 'F' 'A' 'P' 
  for(i=0;i<12;i++)
    {
    switch(i%4)
      {
#define X (4)
#define Y (191)
      case 0:
        Partial_Update_Display_Area(X, // x location, in bytes
                                    Y, // y location in pixels
                                    LETTER_WIDTH_PIXELS,
                                    LETTER_HEIGHT_PIXELS,
                                    Mono_Letter_C);
        break;
      case 1:
        Partial_Update_Display_Area(X, // x location, in bytes
                                    Y, // y location in pixels
                                    LETTER_WIDTH_PIXELS,
                                    LETTER_HEIGHT_PIXELS,
                                    Mono_Letter_F);
        break;
      case 2:
        Partial_Update_Display_Area(X, // x location, in bytes
                                    Y, // y location in pixels
                                    LETTER_WIDTH_PIXELS,
                                    LETTER_HEIGHT_PIXELS,
                                    Mono_Letter_A);
        break;
      case 3:
        Partial_Update_Display_Area(X, // x location, in bytes
                                    Y, // y location in pixels
                                    LETTER_WIDTH_PIXELS,
                                    LETTER_HEIGHT_PIXELS,
                                    Mono_Letter_P);
        break;
#undef X
#undef Y
      }
    delay(200);
    }

  //Load the background inage into both layers of the display
  //controller's RAM
  Partial_Update_Init_Background(WIDTH_PIXELS,  //X size
                                 HEIGHT_PIXELS, //Y size
                                 Image_0);      //image data

  // Swap out the 5 elements in the bottom portion of the screen 
  for(i=0;i<7;i++)
    {
    switch(i%7)
      {
      case 0:
        Partial_Update_Display_Area(0,   // x location, in bytes
                                    100, // y location in pixels
                                    WIDTH_PIXELS_PARTIAL,
                                    HEIGHT_PIXELS_PARTIAL,
                                    Image_1_Partial);
        break;
      case 1:
        Partial_Update_Display_Area(0,   // x location, in bytes
                                    100, // y location in pixels
                                    WIDTH_PIXELS_PARTIAL,
                                    HEIGHT_PIXELS_PARTIAL,
                                    Image_2_Partial);
        break;
      case 2:
        Partial_Update_Display_Area(0,   // x location, in bytes
                                    100, // y location in pixels
                                    WIDTH_PIXELS_PARTIAL,
                                    HEIGHT_PIXELS_PARTIAL,
                                    Image_3_Partial);
        break;
      case 3:
        Partial_Update_Display_Area(0,   // x location, in bytes
                                    100, // y location in pixels
                                    WIDTH_PIXELS_PARTIAL,
                                    HEIGHT_PIXELS_PARTIAL,
                                    Image_4_Partial);
        break;
      case 4:
        Partial_Update_Display_Area(0,   // x location, in bytes
                                    100, // y location in pixels
                                    WIDTH_PIXELS_PARTIAL,
                                    HEIGHT_PIXELS_PARTIAL,
                                    Image_5_Partial);
        break;
      case 5:
        Partial_Update_Display_Area(0,   // x location, in bytes
                                    100, // y location in pixels
                                    WIDTH_PIXELS_PARTIAL,
                                    HEIGHT_PIXELS_PARTIAL,
                                    Image_6_Partial);
        break;
      case 6:
        Partial_Update_Display_Area(0,   // x location, in bytes
                                    100, // y location in pixels
                                    WIDTH_PIXELS_PARTIAL,
                                    HEIGHT_PIXELS_PARTIAL,
                                    Image_7_Partial);
        break;
      }
    delay(1000);
    }
  ePaper_POWEROFF();
  }
//===========================================================================
void setup(void)
  {
  //Debug port / Arduino Serial Monitor
  Serial.begin(115200);
  
  // Configure the pin directions
  pinMode(EPD_BUSY, INPUT);
  pinMode(EPD_RESET , OUTPUT);   
  pinMode(EPD_CS, OUTPUT);     
  pinMode(EPD_DC, OUTPUT);     
  pinMode(EPD_BUSSEL, OUTPUT);
  // Set output pins' default state
  digitalWrite(EPD_RESET, HIGH);   //active low
  digitalWrite(EPD_CS, HIGH);      //active low
  digitalWrite(EPD_DC, HIGH);      //active low 
  digitalWrite(EPD_BUSSEL, LOW);   // 4 wire mode
  
  //Set up SPI interface
  SPI.beginTransaction(SPISettings(500000, MSBFIRST, SPI_MODE0));
  SPI.begin();
  //Initialize the display controller hardware 
  ePaper_Init();
  }
//=============================================================================
void loop()
  {
  // Show the logo from Images_for_CFAP122250A00213.h
  Show_Full_Screen_Image(WIDTH_PIXELS,  //X size
                         HEIGHT_PIXELS, //Y size
                         Image_0);      //image data
                         
  delay(1000);

  //Loop through some smaller images to show off the partial update feature.
  Partial_Update_Letters_Demo();
  delay(2500);
  }
//=============================================================================
